package com.nisum.address.controller;

import java.util.Optional;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.nisum.address.dto.AddressRequest;
import com.nisum.address.model.Address;
import com.nisum.address.service.AddressService;

import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;

@RestController
@RequestMapping("/api/address")
public class AddressController {
	
	private static final Logger logger = LoggerFactory.getLogger(AddressController.class);
	
	@Autowired
	AddressService addressService;

	@PostMapping
	public Mono<ResponseEntity<Address>> createAddress(@RequestBody AddressRequest addressRequest) {
		
		return addressService.createAddress(addressRequest).map(address -> {
			return new ResponseEntity<Address>(address, HttpStatus.CREATED);
		});
		 
		/*
		 * return Optional.ofNullable(addressRequest).orElse(addressRequest).flatMap(
		 * monoAddress -> { return addressService.createAddress(monoAddress).map(address
		 * -> { return new ResponseEntity<Address>(address, HttpStatus.CREATED); }); });
		 */
				
	}
	
	@GetMapping("/{profileId}")
	public Mono<ResponseEntity<Address>> getAddressByProfileId(@PathVariable("profileId") String profileId) {
		return addressService.getProfilesByProfileId(profileId).map(address -> {
			return new ResponseEntity<Address>( address,HttpStatus.OK);
		});

	}
	
	@DeleteMapping("/{id}")
	public Mono<ResponseEntity<Void>> deleteProfile(@PathVariable("id") String id) {
		return addressService.deleteProfileById(id).map(pofile -> {
			return new ResponseEntity<Void>(HttpStatus.NO_CONTENT);
		});
	}


}
