package com.nisum.address.service;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.nisum.address.dao.AddressRepository;
import com.nisum.address.dto.AddressRequest;
import com.nisum.address.exceptions.ResourceNotFoundException;
import com.nisum.address.exceptions.ServiceException;
import com.nisum.address.model.Address;

import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;

@Service
public class AddressServiceImpl implements AddressService{
	private static final Logger logger = LoggerFactory.getLogger(AddressServiceImpl.class);
	
	@Autowired
	AddressRepository addRepository;

	@Override
	public Mono<Address> createAddress(AddressRequest addressRequest) {
		logger.info("createAddress started in service");
		Address address = mapDtoToEntity(addressRequest, new Address());
		return addRepository.save(address)
		            .switchIfEmpty(Mono.error(new ServiceException()));
	}

	private Address mapDtoToEntity(AddressRequest addressRequest, Address address) {
		address.setProfileId(addressRequest.getProfileId());
		address.setCity(addressRequest.getCity());
		address.setPincode(addressRequest.getPincode());
		address.setState(addressRequest.getState());
		return address;
	}

	@Override
	public Mono<Address> getProfilesByProfileId(String profileId) {
		return addRepository.getAddressByProfileId(profileId).log()
				.switchIfEmpty(Mono.error(new ResourceNotFoundException("Profile Id Not Available"))).log();
	}

	@Override
	public Mono<Void> deleteProfileById(String id) {
		return addRepository.findById(id)
				.switchIfEmpty(Mono.error(new ResourceNotFoundException("Profile Id Not Available")))
				.flatMap(profile ->{
					return addRepository.deleteById(id);
				});
	}

}
