package com.nisum.address;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertNotEquals;
import static org.junit.jupiter.api.Assertions.assertNotNull;

import java.util.List;

import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.data.mongo.DataMongoTest;
import org.springframework.test.context.ActiveProfiles;

import com.nisum.address.dao.AddressRepository;
import com.nisum.address.model.Address;

import lombok.var;
import reactor.test.StepVerifier;

@DataMongoTest
@ActiveProfiles("test")
public class AddressRepositoryTest {
	
	@Autowired
	AddressRepository addressRepository;
	
	@BeforeEach
	void setUp() {
		
		var addressInfo = List.of(new Address("1","Pr123","784567","BBSR","ODISHA"),
				new Address("2","RE123","73490","Hyderabad","telengana"),
				new Address("3","DR3","033567","saltlake","kolkata"));
		
		addressRepository.saveAll(addressInfo).blockLast();
		
	}
	
	@AfterEach
	void tearDown() {
		addressRepository.deleteAll().block();
	}
	
	
	@Test
	void findAll() {
		
		
		var addressFlux = addressRepository.findAll().log();
		
		StepVerifier.create(addressFlux)
				.expectNextCount(3)
				.verifyComplete();
		
	}
	
	@Test
	void findById() {
		
		
		var addressMono = addressRepository.findById("2").log();
		
		StepVerifier.create(addressMono)
				.assertNext(addressInfo ->{
					assertEquals("Hyderabad", addressInfo.getCity());
				})
				.verifyComplete();
		
	}
	
	@Test
	void saveAddress() {
		
		
		var addressMono = addressRepository.save(new Address(null,"Pr123","784567","BBSR","ODISHA")).log();
		
		StepVerifier.create(addressMono)
				.assertNext(addressInfo ->{
					assertNotNull(addressInfo.getId());
					assertEquals("BBSR", addressInfo.getCity());
				})
				.verifyComplete();
		
	}
	
	@Test
	void updateAddress() {
		
		var addressMono = addressRepository.findById("2").block();
		addressMono.setCity("Delhi");
		var addressUpdateMono = addressRepository.save(addressMono).log();
		
		StepVerifier.create(addressUpdateMono)
				//.expectNextCount(3)
				.assertNext(addressInfo ->{
					assertNotNull(addressInfo.getId());
					assertEquals("Delhi", addressInfo.getCity());
				})
				.verifyComplete();
		
	}
	
	@Test
	void deleteAdress() {
		
		
		 addressRepository.deleteById("1").block();
		 var addressFlux = addressRepository.findAll().log();
		
		StepVerifier.create(addressFlux)
				.expectNextCount(2)
				.verifyComplete();
		
	}
	

	@Test
	void getAdressByProfileId() {
		 var addressMono= addressRepository.getAddressByProfileId("DR3").log();
		 
		StepVerifier.create(addressMono)
				.assertNext(address->{
					assertEquals("saltlake", address.getCity());
				})
				.verifyComplete();
		
	}

}
